/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.html.demo.chess;

/** Common chess rules.
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
class Rules {
    static void computeAccessible(Board b, Square s) {
        for (Row r : b.getRows()) {
            for (Square ts : r.getColumns()) {
                ts.setAccessible(false);
            }
        }
        if (s == null) {
            return;
        }
        
        switch (s.getPiece()) {
            case BISHOP: 
                moveBishop(b, s);
                break;
            case KING:
                computeAccessible(b, s, 1, 1, 1);
                computeAccessible(b, s, 1, -1, 1);
                computeAccessible(b, s, -1, -1, 1);
                computeAccessible(b, s, -1, 1, 1);
                computeAccessible(b, s, 1, 0, 1);
                computeAccessible(b, s, 0, -1, 1);
                computeAccessible(b, s, 0, 1, 1);
                computeAccessible(b, s, -1, 0, 1);
                break;
            case ROCK:
                moveRock(b, s);
                break;
            case QUEEN:
                moveRock(b, s);
                moveBishop(b, s);
                break;
            case KNIGHT:
                computeAccessible(b, s, 2, 1, 1);
                computeAccessible(b, s, 2, -1, 1);
                computeAccessible(b, s, -2, -1, 1);
                computeAccessible(b, s, -2, 1, 1);
                computeAccessible(b, s, 1, 2, 1);
                computeAccessible(b, s, -1, 2, 1);
                computeAccessible(b, s, -1, -2, 1);
                computeAccessible(b, s, 1, -2, 1);
                break;
            case PAWN:
                pawns(b, s);
                break;
        }
    }

    private static void moveRock(Board b, Square s) {
        computeAccessible(b, s, 1, 0, 8);
        computeAccessible(b, s, 0, -1, 8);
        computeAccessible(b, s, -1, 0, 8);
        computeAccessible(b, s, 0, 1, 8);
    }

    private static void moveBishop(Board b, Square s) {
        computeAccessible(b, s, 1, 1, 8);
        computeAccessible(b, s, 1, -1, 8);
        computeAccessible(b, s, -1, -1, 8);
        computeAccessible(b, s, -1, 1, 8);
    }
    
    private static void computeAccessible(
        Board b, Square s, int dx, int dy,
        int limit
    ) {
        int x = s.getX();
        int y = s.getY();
        
        while (limit-- > 0) {
            x += dx;
            y += dy;
            Square next = BoardModel.findSquare(b, (char)x, y);
            if (next == null) {
                break;
            }
            if (next.getPieceColor() == s.getPieceColor()) {
                break;
            }
            next.setAccessible(true);
            if (next.getPieceColor() != null) {
                break;
            }
        }
    }
    
    private static void pawns(Board b, Square s) {
        final boolean white = s.getPieceColor() == BoardModel.ColorType.WHITE;
        int dy = white ? 1 : -1;
        Square step = BoardModel.findSquare(b, (char)s.getX(), s.getY() + dy);
        if (step != null && step.getPiece() == null) {
            step.setAccessible(true);
            if ((s.getY() == 2 && white) || (s.getY() == 7 && !white)) {
                Square nextSTep = BoardModel.findSquare(b, (char)s.getX(), step.getY() + dy);
                if (nextSTep != null && step.getPiece() == null) {
                    nextSTep.setAccessible(true);
                }
            }
        }
    }
}
